<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://www.centralpay.eu/
 * @since             1.0.0
 * @package           cpay
 *
 * @wordpress-plugin
 * Plugin Name:       CentralPay
 * Plugin URI:        http://wordpress.org/plugins/cpay/
 * Description:       Réalisez des paiements par carte de crédit avec CentralPay.
 * Version:           2.1.1
 * Author:            CentralPay
 * Author URI:        http://www.centralpay.eu/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       cpay
 * Domain Path:       /languages
 * Slug:              cpay
 */

if (!defined('WPINC')) {
    die;
}


if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {


    function activateCpay()
    {
        require_once plugin_dir_path(__FILE__) . 'includes/class-cpay-activator.php';
        CpayActivator::activate();
    }


    function deactivateCpay()
    {
        require_once plugin_dir_path(__FILE__) . 'includes/class-cpay-deactivator.php';
        CpayDeactivator::deactivate();
    }

    require plugin_dir_path(__FILE__) . 'includes/class-cpay-wp-db.php';


    register_activation_hook(__FILE__, 'activateCpay');
    register_deactivation_hook(__FILE__, 'deactivateCpay');

    /*
     * This action hook registers our PHP class as a WooCommerce payment gateway
     */
    add_filter('woocommerce_payment_gateways', 'cpay_add_gateway_class');
    function cpay_add_gateway_class($gateways)
    {
        $gateways[] = 'WC_CPay_Gateway'; // your class name is here
        return $gateways;
    }

    add_action('plugins_loaded', 'set_local');

    function set_local()
    {
        require plugin_dir_path(__FILE__) . 'includes/class-cpay-config.php';
        CpayConfig::set_local();
    }

    function newLog($log_msg) {
        $log_dirpath = plugin_dir_path(__FILE__);

        $log_file_path = $log_dirpath. 'error.log';

        // if you don't add `FILE_APPEND`, the file will be erased each time you add a log
        file_put_contents($log_file_path, date("y-m-d H:i:s.")." ".$log_msg. "\n", FILE_APPEND);
    }

    /*
     * The class itself, please note that it is inside plugins_loaded action hook
     */
    add_action('plugins_loaded', 'cpay_init_gateway_class');
    function cpay_init_gateway_class()
    {

        if (!class_exists('CpayConfig')) {
            require plugin_dir_path(__FILE__) . 'includes/class-cpay-config.php';
        }
        if (!class_exists('Cpay')) {
            require plugin_dir_path(__FILE__) . 'includes/class-cpay.php';
        }

        class WC_CPay_Gateway extends WC_Payment_Gateway
        {

            public function __construct()
            {
                $this->id = 'cpay'; // payment gateway plugin ID
                $this->icon = plugin_dir_url(__FILE__) . 'images/logo-centralpay.png'; // URL of the icon that will be displayed on checkout page near your gateway name
                $this->has_fields = false; // in case you need a custom credit card form
                $this->method_title = __('CentralPay', 'cpay');
                $this->method_description = __('CentralPay allow you to pay online.', 'cpay');
                $this->order_button_text = __('Proceed to payment', 'cpay');
                $this->title = __('Payed with card from CentralPay', 'cpay');

                $this->supports = array(
                    'products'
                );

                $this->init_form_fields();

                $this->init_settings();

                if ($this->get_option('mode_test') === 'yes') {
                    CpayConfig::$TESTMODE = true;
                }
                $this->urlApi = CpayConfig::getApiUrl();

                $this->login_cpay = $this->get_option('login_cpay');
                $this->password_cpay = $this->get_option('password_cpay');
                $this->merchant_public_key = $this->get_option('merchant_public_key');
                $this->point_of_sale_id = $this->get_option('point_of_sale_id');
                $this->oneclick = $this->get_option('one_click');

                if (empty($this->get_option("salt"))) {
                    $this->update_option('salt', CpayConfig::generateSalt());
                }

                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
                add_action('woocommerce_api_cpay_paid', array($this, 'webhook'));
                add_action('woocommerce_api_cpay_validation', array($this, 'pr_validation'));
            }

            public function init_form_fields()
            {
                $this->form_fields = include('includes/settings-cpay.php');
            }

            public function process_payment($order_id)
            {

                global $woocommerce;
                require plugin_dir_path(__FILE__) . 'includes/class-cpay-api.php';
                if (!empty($order_id) && is_int($order_id)) {
                    $userpwd = $this->login_cpay . ":" . $this->password_cpay;

                    $order = wc_get_order($order_id);
                    $reference=  $this->merchant_public_key."-".$order_id;
                    $amount = $order->get_total() * 100;

                    $urlApiCheckPr = $this->urlApi . "paymentRequest?merchantPaymentRequestId=".$reference;

                    $requeteCheckPr = CpayApi::curl_get_api($urlApiCheckPr, $userpwd, [], "GET");

                    if ($requeteCheckPr['http_code'] == 200) {
                        $reponseCheck = $requeteCheckPr['reponse'];
                        if (!empty($reponseCheck->totalCount)) {

                            foreach ($reponseCheck->list as $paymentRequest) {

                                $amountPr = $paymentRequest->totalAmount;
                                $requete['reponse'] = $paymentRequest;
                                if ($amountPr == $amount) {
                                    $urlredirect = Cpay::getUrlSmartForm($order_id, $requete);
                                    if (!empty($urlredirect)) {
                                        return array(
                                            'result' => 'success',
                                            'redirect' => $urlredirect,
                                        );
                                    }
                                    newLog("[ERROR]cpay.php l.".__LINE__." GET URL FORM EMPTY, {asked(int) order_id : sent(".gettype($order_id).") ".$order_id.", requete : ".json_encode($requete)."}");
                                }
                            }
                        }
                        else {
                            var_dump( "empty");
                        }
                    }


                    $email = $order->get_billing_email();

                    $breakdown = ["amount"=>$amount,"email"=>$email];
                    $args = array(
                        'paymentMethod[]' => 'TRANSACTION',
                        'currency' => 'EUR',
                        'totalAmount' => $amount,
                        'pointOfSaleId' => $this->point_of_sale_id,
                        'merchantPaymentRequestId' => $reference
                    );
                    if ($this->oneclick === "yes") {
                        $checkUser = DB::getCustomerId(get_current_user_id(),DB_NAME, DB_PASSWORD);
                        if (!empty($checkUser)) {
                            $breakdown['customerId'] = $checkUser;
                        }
                        else {
                            $args['createCustomer'] = 'true';
                        }
                    }

                    $args['breakdown[]'] = json_encode($breakdown);

                    $urlApi = $this->urlApi . "paymentRequest";
                    $requete = CpayApi::curl_get_api($urlApi, $userpwd, $args);

                    if ($requete['http_code'] == 200) {
                        $urlredirect = Cpay::getUrlSmartForm($order_id, $requete);
                        if (!empty($urlredirect)) {
                            return array(
                                'result' => 'success',
                                'redirect' => $urlredirect,
                            );
                        }
                        wc_add_notice(__('An error occurred during the processing of your order', 'cpay'), 'error');
                        newLog("[ERROR]cpay.php l.".__LINE__." GET URL FORM EMPTY, {asked(int) order_id : sent(".gettype($order_id).") ".$order_id.", requete : ".json_encode($requete)."}");

                        return;
                    } else {
                        wc_add_notice(__('Service connection error', 'cpay'), 'error');
                        newLog("[ERROR]cpay.php l.".__LINE__." CALL API, ['http_code' =>".$requete['http_code']."] {{asked(string) url : sent(".gettype($urlApi).") ".$urlApi."},{asked(array) arg : sent(".gettype($args).")".json_encode($args)."}}");

                        return;
                    }
                }
                wc_add_notice(__('An error occurred during the processing of your order', 'cpay'), 'error');
                newLog("[ERROR]cpay.php l.".__LINE__." PROCESS_PAYMENT \"order_id require\", {asked(int) order_id : sent(".gettype($order_id).") " .$order_id.")}");

                return;
            }

            public function webhook()
            {
                require plugin_dir_path(__FILE__) . 'includes/class-cpay-api.php';

                if (!empty($_GET['idOrder'])) {
                    $urlApi = $this->urlApi . "paymentRequest/".$_GET['paymentRequestId'];
                    $userpwd = $this->login_cpay . ":" . $this->password_cpay;

                    if ($this->oneclick === "yes") {
                        $this->createUser($urlApi,$userpwd);
                    }

                    $orderId = Cpay::decrypteData($_GET['idOrder']);
                    $order = $this->validpayment($orderId);
                    if (!empty($order)) {
                        wp_redirect($this->get_return_url($order));
                        exit();
                    }

                    newLog("[ERROR]cpay.php l.".__LINE__." ID ORDER NOT MATCH, {sent GET['idOrder']=> ".$_GET['idOrder']."}");
                    wc_add_notice( __("An error occurred while processing your request","cpay"), 'notice' );
                    wp_redirect( get_permalink( get_option('woocommerce_myaccount_page_id') ) );
                    exit();
                }
                wc_add_notice( __("An error occurred while processing your request","cpay"), 'notice' );
                newLog("[ERROR]cpay.php l.".__LINE__." WEBHOOK ID ORDER IS EMPTY \"GET[idOrder] require\", {asked(int) idOrder : sent(".gettype($_GET['idOrder'])." ".$_GET['idOrder'].")}");
                // Redirection vers la page compte de l'utilisateur
                wp_redirect( get_permalink( get_option('woocommerce_myaccount_page_id') ) );
            }

            public function pr_validation()
            {
                require plugin_dir_path(__FILE__) . 'includes/class-cpay-api.php';
                if (!empty(json_decode(file_get_contents('php://input'))->object)) {
                    if (!empty(json_decode(file_get_contents('php://input'))->object->transactionStatus) && json_decode(file_get_contents('php://input'))->object->transactionStatus === "SUCCESS") {
                        $paymentRequest = json_decode(file_get_contents('php://input'))->object;
                        $marchandTransactionId = $paymentRequest->merchantTransactionId;

                        $arrayTransactionId = explode("-", $marchandTransactionId);
                        $idMerchand = $arrayTransactionId[0];
                        $orderId = end($arrayTransactionId);
                        $order = wc_get_order($orderId);

                        if (!empty($orderId && $order->status == "pending") && $idMerchand === $this->merchant_public_key) {

                            $urlApi = $this->urlApi . "paymentRequest/".$paymentRequest->paymentRequestId;

                            $userpwd = $this->login_cpay . ":" . $this->password_cpay;

                            if ($this->oneclick === "yes") {
                                $this->createUser($urlApi,$userpwd);
                            }

                            $order = $this->validpayment($orderId);

                            if (!empty($order)) {
                                echo "order ". $orderId." status : ".$order->status;
                                exit();
                            }
                            newLog("[ERROR]cpay.php l.".__LINE__." ID ORDER NOT MATCH, {sent GET['idOrder']=> ".$orderId."}");
                            wc_add_notice( __("An error occurred while processing your request","cpay"), 'notice' );
                            // Redirection vers la page compte de l'utilisateur
                            wp_redirect( get_permalink( get_option('woocommerce_myaccount_page_id') ) );
                            exit();
                        }
                        wc_add_notice( __("An error occurred while processing your request","cpay"), 'notice' );
                        newLog("[ERROR]cpay.php l.".__LINE__." WEBHOOK transaction id IS INVALID : ".$arrayTransactionId);
                        // Redirection vers la page compte de l'utilisateur
                        wp_redirect( get_permalink( get_option('woocommerce_myaccount_page_id') ) );
                        exit();

                    }
                }
                die();
            }

            public function createUser($urlApi,$userpwd) {
                $checkUser = DB::getCustomerId(get_current_user_id(),DB_NAME, DB_PASSWORD);

                if (empty($checkUser)) {

                    $requete = CpayApi::curl_get_api($urlApi,$userpwd,[],'GET');

                    if (!empty($requete['reponse']->breakdowns) && !empty($requete['reponse']->breakdowns[0]->customerId)) {
                        $transactionData = [];
                        $transactionData['customerId'] = $requete['reponse']->breakdowns[0]->customerId;
                        $transactionData['userId'] = get_current_user_id();
                        DB::createCustomer($transactionData, DB_USER,DB_PASSWORD);
                    }
                    else {
                        newLog("[ERROR]cpay.php l.".__LINE__." GET CUSTOMERID TRANSACTION BAD VALUE BREAKDOWS/CUSTOMERID, {{asked(array) requete['reponse']->breakdown : sent(".gettype($requete['reponse']->breakdowns).")".json_encode($requete['reponse']->breakdowns)."},{asked(string) customerID : sent(".gettype($requete['reponse']->breakdowns[0]->customerId).") ".$requete['reponse']->breakdowns[0]->customerId."}");
                    }
                }
            }

            public function validpayment ($orderId) {
                $order = wc_get_order($orderId);
                if (!empty($order)) {
                    $order->payment_complete();
                    $order->reduce_order_stock();
                    return $order;
                }
            }
        }
    }

}
